/***************************************************************************
 *   Copyright (C) 2012 by santiago González                               *
 *   santigoro@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/

#ifndef COMPONENTSELECTOR_H
#define COMPONENTSELECTOR_H

#include <QDir>
#include <QTreeWidget>

#include "managecomps.h"
#include "itemlibrary.h"

class ComponentSelector : public QTreeWidget
{
    Q_OBJECT

    public:
        ComponentSelector( QWidget* parent );
        ~ComponentSelector();

 static ComponentSelector* self() { return m_pSelf; }

        void LoadCompSetAt( QDir compSetDir );

        QString getXmlFile( QString compName ) { return m_xmlFileList[ compName ]; }

        void search( QString filter );

        void hideFromList( QTreeWidgetItem* item, bool hide );

        QTreeWidgetItem* getCategory( QString category );
        QStringList getCategories(){ return m_categories.keys(); }

        void setShortCuts( QHash<QString, QString> s ) { m_shortCuts = s; }
        QHash<QString, QString> getShortCuts() {
            return m_shortCuts; }

    private slots:
        void slotItemClicked( QTreeWidgetItem* item, int );
        void slotContextMenu( const QPoint& );
        void slotManageComponents();

    private:
 static ComponentSelector* m_pSelf;

        void loadXml( const QString &setFile );
        void addItem( QString caption, QTreeWidgetItem* catItem, QString icon, QString type );
        void LoadLibraryItems();

        QTreeWidgetItem* addCategory( QString nameTr, QString name, QString parent, QString icon );

        QStringList m_components;
        QHash<QString, QTreeWidgetItem*> m_categories;
        QHash<QString, QString> m_catTr;

        QHash<QString, QString> m_xmlFileList;
        QHash<QString, QString> m_shortCuts;

        manCompDialog m_mcDialog;

        ItemLibrary m_itemLibrary;
};

#endif
