/***************************************************************************
 *   Copyright (C) 2020 by santiago González                               *
 *   santigoro@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/

#include "i51usart.h"
#include "usarttx.h"
#include "usartrx.h"
#include "mcutimer.h"
#include "e_mcu.h"
#include "i51interrupt.h"
#include "datautils.h"

#define SCON *m_scon

I51Usart::I51Usart( eMcu* mcu, QString name, int number )
        : McuUsart( mcu, name, number )
{
    m_stopBits = 1;
    m_dataMask = 0xFF;
    m_parity   = parNONE;

    m_timer1 = mcu->getTimer( "TIMER1" );

    m_bit9Tx = getRegBits( "TB8", mcu );
    m_bit9Rx = getRegBits( "RB8", mcu );
    m_REN    = getRegBits( "REN", mcu );
    m_SM     = getRegBits( "SM1,SM0", mcu );
    m_SMOD   = getRegBits( "SMOD", mcu );
}
I51Usart::~I51Usart(){}

void I51Usart::configureA(uint8_t newSCON ) //SCON
{
    uint8_t mode = getRegBitsVal( newSCON, m_SM );
    if( mode == m_mode ) return;
    m_mode = mode;
    m_sender->enable( true );
    m_receiver->enable( getRegBitsBool( newSCON, m_REN ) );

    uint8_t SMOD = getRegBitsVal( m_SMOD );
    m_div = 16*(2-SMOD);

    m_useTimer = false;

    switch( mode )
    {
        case 0:             // Synchronous MCU Clock
            /// TODO //setPeriod( m_mcu->psPerClk()/m_div  );// Fixed baudrate 32 or 64
            m_dataBits = 8;
            break;
        case 1:             // Asynchronous Timer1
            m_useTimer = true;
            m_dataBits = 8;
            break;
        case 2:             // Asynchronous MCU Clock
            setPeriod( m_mcu->psPerClk()/m_div );// Fixed baudrate 32 or 64
            m_dataBits = 9;
            break;
        case 3:             // Asynchronous Timer1
            m_useTimer = true;
            m_dataBits = 9;
            break;
    }
    m_framesize = 1+m_dataBits+m_parity+m_stopBits;

    I51T1Int* t1Int = static_cast<I51T1Int*>( m_timer1->getInterrupt() ); //  .connect( this, &I51Usart::step );
    if( m_useTimer )
    {
        t1Int->setUsart( this );
        setPeriod( 0 );
        m_counter = 0;
    }
    else t1Int->setUsart( NULL );
}

void I51Usart::step()
{
    if( m_useTimer && ++m_counter < m_div ) return;

    m_counter = 0;
    m_sender->runEvent();
    m_receiver->runEvent();

}

uint8_t I51Usart::getBit9()
{
    return getRegBitsVal( m_bit9Tx );
}

void I51Usart::setBit9( uint8_t bit )
{
    writeRegBits( m_bit9Rx, bit );
}

